#include "mbed.h"

#define TRIGGER_VOLTAGE_MV 1450
#define USE_HYSTERESIS //comment this out to use a debounce delay instead of hysteresis 
#define HYSTERESIS_TRESHOLD_MV 500 //Sensor voltage has to drop below this value to enable next signal. Should prevent multiple triggers caused by voltage fluctuations
#define FILTER_SAMPLES 5
#define SIGNAL_LENGTH_MS 10
#define DEBOUNCE_WAIT_MS 300 //only used when hysteresis is not enabled. Wait a fixed amount of milliseconds to prevent multiple triggers

AnalogIn sensor(A0);
DigitalOut led(LED1);
DigitalOut signal(D2);
//InterruptIn user(USER_BUTTON); //no user button on Nucleo F303K8

short buffer[FILTER_SAMPLES];
short bufferPos= 0;

short flashCounter = 0;

bool isInitialized = false;
bool isReady = true;
bool hysteresis = false;

Ticker ledFlasher;
Timeout signalTimeout;
Timeout debounceTimeout;

void flashLED()
{
    if(flashCounter++ > 30)
        ledFlasher.detach();

    led = !led;
}

void reset()
{
    isReady = true;
}

void endSignal()
{
    led = 0;
    signal = 0;
}

void startSignal()
{
    led = 1;
    signal = 1;
    signalTimeout.attach_us(&endSignal, 1000 * SIGNAL_LENGTH_MS);
#ifndef USE_HYSTERESIS
    debounceTimeout.attach_us(&reset, 1000 * DEBOUNCE_WAIT_MS);
#endif
}

short getAverageVoltage()
{
    short sum;
    for (int i = 0; i < FILTER_SAMPLES; i++)
        sum += buffer[i];

    return sum / FILTER_SAMPLES;
}

int main()
{
    if(!isInitialized) {
        isInitialized = true;
        led = 0;
        signal = 0;
        ledFlasher.attach(&flashLED, 0.03f);
        //user.mode(PullDown);
        //user.fall(&startSignal);
    }

    while(1) {
        buffer[bufferPos++] = sensor.read() * 3300;

        if(bufferPos == FILTER_SAMPLES)
            bufferPos = 0;

#ifdef USE_HYSTERESIS
        short voltage = getAverageVoltage();

        if(voltage <= HYSTERESIS_TRESHOLD_MV)
            hysteresis = false;

        if(voltage >= TRIGGER_VOLTAGE_MV && !hysteresis) {
            startSignal();
            hysteresis = true;
        }
#else
        if(isReady) {
            if(getAverageVoltage() >= TRIGGER_VOLTAGE_MV) {
                startSignal();
                isReady = false;

            }
        }
#endif
    }
}
